import { NgModule } from '@angular/core';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import {
  StoreRouterConnectingModule,
} from '@ngrx/router-store';
import { EffectsModule } from '@ngrx/effects';


import { environment } from '../environments/environment';
import { REDUCERS, META_REDUCERS, EFFECTS } from './store';

@NgModule({
  exports: [
    StoreModule,
  ],
  imports: [
    StoreModule.forRoot(REDUCERS, { metaReducers: META_REDUCERS }),
    /**
     * @ngrx/router-store keeps router state up-to-date in the store.
     */
    StoreRouterConnectingModule.forRoot({
      /*
        They stateKey defines the name of the state used by the router-store reducer.
        This matches the key defined in the map of reducers
      */
      stateKey: 'router',
    }),
    /**
     * Store devtools instrument the store retaining past versions of state
     * and recalculating new states. This enables powerful time-travel
     * debugging.
     *
     * To use the debugger, install the Redux Devtools extension for either
     * Chrome or Firefox
     *
     * See: https://github.com/zalmoxisus/redux-devtools-extension
     */
    StoreDevtoolsModule.instrument({
      name: 'overFlow WebApp DevTools',
      maxAge: 50,
      logOnly: environment.production,
    }),
    EffectsModule.forRoot(EFFECTS),
  ],
  providers: [
  ],

})
export class AppStoreModule { }
