import { Component, Input, OnChanges, SimpleChanges, ViewChildren, QueryList } from '@angular/core';

import { of } from 'rxjs';
import { map, catchError, take } from 'rxjs/operators';

import { Service } from '@overflow/model/discovery';
import { AutoHeightDirective } from '@overflow/commons/ui/directive/auto-height.directive';
import { PingResult } from '@overflow/model/ping';

import { PingService } from '../../../service/ping.service';
import { PingOption } from '@overflow/model/config/ping';

import * as log from '../../../core/util/log';

@Component({
  selector: 'app-infra-detail-service',
  templateUrl: './service.component.html',
  styleUrls: ['./service.component.scss'],
})
export class ServiceComponent implements OnChanges {

  @Input() service: Service;

  @ViewChildren(AutoHeightDirective) autoHeightDirectives: QueryList<AutoHeightDirective>;


  pingWaiting: boolean;
  pingResult: PingResult;
  pingResultStr: string;

  count: number;
  interval: number;
  deadline: number;


  constructor(
    private pingService: PingService
  ) {
    this.pingWaiting = false;
    this.count = 5;
    this.interval = 1;
    this.deadline = 1;
  }

  ngOnChanges(simpleChanges: SimpleChanges): void {
    this.pingResult = null;
  }


  doPing() {
    this.pingWaiting = true;

    const option: PingOption = {
      count: this.count,
      interval: this.interval,
      deadline: this.deadline,
    };

    this.pingService.pingService(this.service, option)
      .pipe(
        map((pingResult: PingResult) => {
          // log.debug(pingResult);
          if (pingResult) {
            this.pingResult = pingResult;
            this.pingWaiting = false;
          }
        }),
        catchError(error => {
          log.error(error);
          alert('An error has occurred.');
          this.pingWaiting = false;
          return of();
        }),
        take(1)
      )
      .subscribe();
  }

  onTabViewChange($event) {
    const index = String($event.index);
    this.autoHeightDirectives.map(item => {
      if (item.key === index) {
        item.forceCalculate();
      }
    });
  }

}
