import { Injectable } from '@angular/core';

import { Zone, DiscoverHost, Host, DiscoverPort, Port, DiscoverService } from '@overflow/model/discovery';

import { ProbeService } from './probe.service';
import { DiscoverRequestInfo } from '../core/model';

@Injectable()
export class DiscoveryService {

  public constructor(
    private probeService: ProbeService,
  ) {
  }

  public discover(discoverRequestInfo: DiscoverRequestInfo): void {
    if (null !== discoverRequestInfo.zone && null !== discoverRequestInfo.discoverHost) {
      this.discoverHost(
        discoverRequestInfo.requesterID,
        discoverRequestInfo.zone,
        discoverRequestInfo.discoverHost
      );
    } else if (null !== discoverRequestInfo.host && null !== discoverRequestInfo.discoverPort) {
      this.discoverPort(
        discoverRequestInfo.requesterID,
        discoverRequestInfo.host,
        discoverRequestInfo.discoverPort
      );
    } else if (null !== discoverRequestInfo.port && null !== discoverRequestInfo.discoverService) {
      this.discoverService(
        discoverRequestInfo.requesterID,
        discoverRequestInfo.port,
        discoverRequestInfo.discoverService
      );
    } else {
      return;
    }
  }

  public discoverHost(requesterID: string, zone: Zone, discoverHost: DiscoverHost): void {
    this.probeService.send('DiscoveryService.DiscoverHost', requesterID, zone, discoverHost);
  }

  public discoverPort(requesterID: string, host: Host, discoverPort: DiscoverPort): void {
    this.probeService.send('DiscoveryService.DiscoverPort', requesterID, host, discoverPort);
  }

  public discoverService(requesterID: string, port: Port, discoverService: DiscoverService): void {
    this.probeService.send('DiscoveryService.DiscoverService', requesterID, port, discoverService);
  }

  public discoverStop(requesterID: string, requestID: string): void {
    this.probeService.send('DiscoveryService.DiscoverStop', requesterID, requestID);
  }
}
