import { Type } from '@angular/core';
import {
  Selector,
  createSelector,
  combineReducers
} from '@ngrx/store';

import * as ConfigStore from './config';
import * as RequestStore from './request';


export interface State {
  config: ConfigStore.State;
  request: RequestStore.State;
}

export const EFFECTS: Type<any>[] = [
  ConfigStore.ConfigEffects,
  RequestStore.RequestEffects,
];

export const REDUCER = combineReducers({
  config: ConfigStore.reducer,
  request: RequestStore.reducer,
});

export function getSelectors<S>(selector: Selector<any, State>) {
  return {
    ConfigSelector: ConfigStore.getSelectors(createSelector(
      selector,
      (state: State) => state.config,
    )),
    RequestSelector: RequestStore.getSelectors(createSelector(
      selector,
      (state: State) => state.request,
    )),
  };
}
