import { Type } from '@angular/core';
import {
  ActionReducer,
  ActionReducerMap,
  MetaReducer,
} from '@ngrx/store';

import * as fromRouter from '@ngrx/router-store';
import { storeFreeze } from 'ngrx-store-freeze';

import * as DiscoveryStore from './discovery';
import * as EnvironmentStore from './environment';
import * as InfraStore from './infra';
import * as UIStore from './ui';

import { environment } from '../../environments/environment';

export const EFFECTS: Type<any>[] = [
  ...DiscoveryStore.EFFECTS,
  ...EnvironmentStore.EFFECTS,
  ...InfraStore.EFFECTS,
  ...UIStore.EFFECTS,
];

export const REDUCERS: ActionReducerMap<any> = {
  router: fromRouter.routerReducer,
  discovery: DiscoveryStore.REDUCER,
  environment: EnvironmentStore.REDUCER,
  infra: InfraStore.REDUCER,
  ui: UIStore.REDUCER,
};

// console.log all actions
export function logger(reducer: ActionReducer<State>): ActionReducer<State> {
  return function (state: State, action: any): State {
    const result = reducer(state, action);
    console.groupCollapsed(action.type);
    console.log('prev state', state);
    console.log('action', action);
    console.log('next state', result);
    console.groupEnd();

    return result;
  };
}

export const META_REDUCERS: MetaReducer<State>[] = !environment.production
  ? [logger]
  : [];

export interface State {
  router: fromRouter.RouterReducerState;
  discovery: DiscoveryStore.State;
  environment: EnvironmentStore.State;
  infra: InfraStore.State;
  ui: UIStore.State;
}

export const DiscoverySelector = DiscoveryStore.getSelectors(
  (state: State) => state.discovery,
);

export const EnvironmentSelector = EnvironmentStore.getSelectors(
  (state: State) => state.environment,
);

export const InfraSelector = InfraStore.getSelectors(
  (state: State) => state.infra,
);

export const UISelector = UIStore.getSelectors(
  (state: State) => state.ui,
);
