import { ElementRef } from '@angular/core';
import { Action } from '@ngrx/store';
import { Host, Port, Service } from '@overflow/model/discovery';
import { InfraDisplayType } from '../../../core/type';

export enum ActionType {
  SetHosts = '[app.infra] SetHosts',
  SetPorts = '[app.infra] SetPorts',
  SetServices = '[app.infra] SetServices',

  ChangeInfraDisplayType = '[app.infra] ChangeInfraDisplayType',
  RefreshInfraDisplay = '[app.infra] RefreshInfraDisplay',
  ChangeSelectedInfra = '[app.infra] ChangeSelectedInfra',
  ChangeInfraDisplayElementRef = '[app.infra] ChangeInfraDisplayElementRef',
}

export class SetHosts implements Action {
  readonly type = ActionType.SetHosts;

  constructor(public payload: Host[]) { }
}

export class SetPorts implements Action {
  readonly type = ActionType.SetPorts;

  constructor(public payload: Port[]) { }
}

export class SetServices implements Action {
  readonly type = ActionType.SetServices;

  constructor(public payload: Service[]) { }
}

export class ChangeInfraDisplayType implements Action {
  readonly type = ActionType.ChangeInfraDisplayType;

  constructor(public payload: { infraDisplayType: InfraDisplayType }) { }
}

export class RefreshInfraDisplay implements Action {
  readonly type = ActionType.RefreshInfraDisplay;

  constructor(public payload: boolean) { }
}

export class ChangeSelectedInfra implements Action {
  readonly type = ActionType.ChangeSelectedInfra;

  constructor(public payload: { group: string, infra: Host | Port | Service }) { }
}

export class ChangeInfraDisplayElementRef implements Action {
  readonly type = ActionType.ChangeInfraDisplayElementRef;

  constructor(public payload: { infraDisplayElementRef: ElementRef }) { }
}

export type Actions =
  | SetHosts
  | SetPorts
  | SetServices
  | ChangeInfraDisplayType
  | RefreshInfraDisplay
  | ChangeSelectedInfra
  | ChangeInfraDisplayElementRef

  ;
