import { ElementRef } from '@angular/core';
import { Selector, createSelector } from '@ngrx/store';
import { Host, Port, Service } from '@overflow/model/discovery';
import { InfraDisplayType } from '../../../core/type';

export interface State {
  hosts: Host[] | null;
  ports: Port[] | null;
  services: Service[] | null;

  infraDisplayType: InfraDisplayType;
  refreshInfraDisplay: boolean;
  selectedInfra: {
    group: string;
    infra: Host | Port | Service;
  } | null;
  infraDisplayElementRef: ElementRef;
}

export const initialState: State = {
  hosts: null,
  ports: null,
  services: null,

  infraDisplayType: InfraDisplayType.NONE,
  refreshInfraDisplay: false,
  selectedInfra: null,
  infraDisplayElementRef: null,
};

export function getSelectors<S>(selector: Selector<any, State>) {
  return {
    selectHosts: createSelector(selector, (state: State) => state.hosts),
    selectPorts: createSelector(selector, (state: State) => state.ports),
    selectServices: createSelector(selector, (state: State) => state.services),
    selectInfraDisplayType: createSelector(selector, (state: State) => state.infraDisplayType),
    selectRefreshInfraDisplay: createSelector(selector, (state: State) => state.refreshInfraDisplay),
    selectSelectedInfra: createSelector(selector, (state: State) => state.selectedInfra),
    selectInfraDisplayElementRef: createSelector(selector, (state: State) => state.infraDisplayElementRef),
  };
}
